@@ -1,6 +1,6 @@
 ;;; Foreign function interface for CLISP
 ;;; Bruno Haible 19.2.1995
-;;; Sam Steingold 1998-2008
+;;; Sam Steingold 1998-2009
 
 #+UNICODE
 (progn
@@ -124,9 +124,11 @@
           '(nil boolean character char uchar short ushort int uint long ulong
             uint8 sint8 uint16 sint16 uint32 sint32 uint64 sint64
             single-float double-float
-            size_t ssize_t      ; see foreign.d:init_ffi()
             c-pointer c-string))
   (setf (gethash c-type *c-type-table*) c-type))
+(dolist (c-type '(size_t ssize_t)) ; see foreign.d:init_ffi()
+  (setf (gethash c-type *c-type-table*)
+        (CS-CL:symbol-name c-type))) ; prefer lower case
 ;; FILE* is used by modern ffi packages
 (setf (gethash '|file| *c-type-table*) 'c-pointer) ; (def-c-type FILE c-pointer)
 
@@ -158,7 +160,7 @@
                 h)
            (eval `(FUNCTION
                    (LAMBDA ,vars
-                    (DECLARE (COMPILE))
+                    (DECLARE (COMPILE ,(intern (string-concat "MAKE-" (symbol-name class)) (symbol-package class))))
                     ,(if (and (setq h (get class 'clos::closclass))
                               (typep h clos::<structure-class>)
                               (setq h (clos::class-kconstructor h)))
@@ -395,11 +397,11 @@
                           (warn (TEXT "~S argument ~S is not a pointer in ~S")
                                 argmode argtype whole))
                         (list argtype
-                              (+ (ecase argmode
+                              (+ (sys::mecase whole argmode
                                    ((:IN :READ-ONLY) 0)
                                    ((:OUT :WRITE-ONLY) ff-flag-out)
                                    ((:IN-OUT :READ-WRITE) ff-flag-in-out))
-                                 (ecase argalloc
+                                 (sys::mecase whole argalloc
                                    (:NONE 0)
                                    (:ALLOCA ff-flag-alloca)
                                    (:MALLOC-FREE ff-flag-malloc-free))))))
@@ -407,7 +409,7 @@
             'simple-vector)
     (+ (let ((rettype (assoc ':return-type alist)))
          (if (cddr rettype)
-           (ecase (third rettype)
+           (sys::mecase whole (third rettype)
              (:NONE 0)
              (:MALLOC-FREE ff-flag-malloc-free))
            0))
@@ -689,7 +691,7 @@
           (let ((module-name (pathname-name *coutput-file*)))
             (make-ffi-module :name module-name
                              :c-name (to-module-name module-name))))
-    (format *coutput-stream* "extern object module__~A__object_tab[];~%"
+    (format *coutput-stream* "extern gcv_object_t module__~A__object_tab[];~%"
             *c-name*)))
 (defun finalize-coutput-file ()
   (when *ffi-module*
@@ -699,11 +701,11 @@
             *c-name* *c-name* *c-name*)
     (let ((count (hash-table-count *object-table*)))
       (if (zerop count)
-        (format *coutput-stream* "object module__~A__object_tab[1];~%~
+        (format *coutput-stream* "gcv_object_t module__~A__object_tab[1];~%~
                 object_initdata_t module__~A__object_tab_initdata[1];~%"
                 *c-name* *c-name*)
         (let ((v (make-array count)))
-          (format *coutput-stream* "object module__~A__object_tab[~D];~%~
+          (format *coutput-stream* "gcv_object_t module__~A__object_tab[~D];~%~
                   object_initdata_t module__~A__object_tab_initdata[~D] = {~%"
                   *c-name* count *c-name* count)
           (dohash (key value *object-table*)
@@ -720,6 +722,10 @@
     (maphash (lambda (type fun-vec)
                (let* ((fun (first fun-vec)) (vec (second fun-vec))
                       (c-decl (to-c-typedecl type fun)))
+                 (when (eq type 'c-string)
+                   ;; avoid warning:
+                   ;; deprecated conversion from string constant to 'char*'
+                   (setq c-decl (string-concat "const " c-decl)))
                  (format *coutput-stream* "~A (int number, int *definedp);~%~
                                            ~A (int number, int *definedp) {
   *definedp=1;~%  switch (number) {~%"
@@ -944,7 +950,7 @@
          (flags (+ (if read-only fv-flag-readonly 0)
                    (let ((alloc (assoc ':alloc alist)))
                      (if (cdr alloc)
-                       (ecase (second alloc)
+                       (sys::mecase whole-form (second alloc)
                          (:NONE 0)
                          (:MALLOC-FREE fv-flag-malloc-free))
                        0))))
@@ -1134,8 +1140,55 @@
          (NOTE-C-CALL-IN ',name ',c-name ',alist ',whole-form))
        ',name)))
 
-(defun convert-to-foreign-C (flags)
-  (if (flag-set-p flags ff-flag-malloc-free) "mallocing" "nomalloc"))
+;; convert-from-foreign & convert-to-foreign inline
+;; foreign.d:convert_from_foreign and foreign.d:convert_to_foreign
+;; for callbacks into lisp.
+;; we inline only a few most common cases - those used in the supplied modules.
+(defun convert-from-foreign (argtype argname)
+  ;; keep in sync with foreign.d:convert_from_foreign
+  (case argtype
+    (nil "NIL")
+    (boolean (format nil "~A ? T : NIL" argname))
+    ;; (character ...) too hairy
+    ((char sint8) (format nil "sint8_to_I(~A)" argname))
+    ((uchar uint8) (format nil "uint8_to_I(~A)" argname))
+    (sint16 (format nil "sint16_to_I(~A)" argname))
+    (uint16 (format nil "uint16_to_I(~A)" argname))
+    (sint32 (format nil "sint32_to_I(~A)" argname))
+    (uint32 (format nil "uint32_to_I(~A)" argname))
+    (sint64 (format nil "sint64_to_I(~A)" argname))
+    (uint64 (format nil "uint64_to_I(~A)" argname))
+    (int (format nil "sint_to_I(~A)" argname))
+    (uint (format nil "uint_to_I(~A)" argname))
+    (long (format nil "slong_to_I(~A)" argname))
+    (ulong (format nil "ulong_to_I(~A)" argname))
+    (single-float (format nil "c_float_to_FF((const ffloatjanus*)&~A)" argname))
+    (double-float (format nil "c_double_to_FF((const dfloatjanus*)&~A)" argname))
+    (c-pointer
+     (let ((addr (format nil "(uintP)(*(void* const *) ~A)" argname)))
+       (format nil "~A == 0 ? NIL : make_faddress(GLO(fp_zero),~A)"
+               addr addr)))
+    (c-string (format nil "~A == NULL ? NIL : asciz_to_string(~A,GLO(foreign_encoding))" argname argname))
+    (t (format nil "convert_from_foreign(~A,&~A)"
+               (object-to-c-value (pass-object argtype)) argname))))
+
+(defun convert-to-foreign (rettype lispobj retaddr flags)
+  ;; keep in sync with foreign.d:convert_to_foreign
+  (case rettype
+    (int (format nil "if (sint_p(~A)) *~A=I_to_sint(~A); else error_sint(~A)"
+                 lispobj retaddr lispobj lispobj))
+    (uint (format nil "if (uint_p(~A)) *~A=I_to_uint(~A); else error_uint(~A)"
+                  lispobj retaddr lispobj lispobj))
+    (long
+     (format nil "if (slong_p(~A)) *~A=I_to_slong(~A); else error_slong(~A)"
+             lispobj retaddr lispobj lispobj))
+    (ulong
+     (format nil "if (ulong_p(~A)) *~A=I_to_ulong(~A); else error_ulong(~A)"
+             lispobj retaddr lispobj lispobj))
+    (t (format nil "convert_to_foreign(~A,~A,~A,&~A)"
+               (object-to-c-value (pass-object rettype)) lispobj retaddr
+               (if (flag-set-p flags ff-flag-malloc-free)
+                   "mallocing" "nomalloc")))))
 
 (defun note-c-call-in (name c-name alist whole) ; ABI
   (when (compiler::prepare-coutput-file)
@@ -1174,9 +1227,8 @@
             (flag-output (logior ff-flag-out ff-flag-in-out)))
         (mapc #'(lambda (argtype argflag argname)
                   (unless (flag-set-p argflag ff-flag-out)
-                    (format *coutput-stream*
-                            "  pushSTACK(convert_from_foreign(~A,&~A));~%"
-                            (object-to-c-value (pass-object argtype)) argname)
+                    (format *coutput-stream* "  pushSTACK(~A);~%"
+                            (convert-from-foreign argtype argname))
                     (incf inargcount))
                   (when (flag-set-p argflag flag-output)
                     (incf outargcount)))
@@ -1184,28 +1236,24 @@
         (format *coutput-stream* "  funcall(~A,~D);~%"
                 (object-to-c-value (pass-object name)) inargcount)
         (unless (eq rettype 'NIL)
-          (format *coutput-stream* " {~%  ~A;~%~:
-  convert_to_foreign(~A,value1,&retval,&~A);~%"
+          (format *coutput-stream* " {~%  ~A;~%  ~A;~%"
                   (to-c-typedecl rettype "retval")
-                  (object-to-c-value (pass-object rettype))
-                  (convert-to-foreign-C flags)))
+                  (convert-to-foreign rettype "value1" "&retval" flags)))
         (let ((outargcount (if (eq rettype 'NIL) 0 1)))
           (mapc #'(lambda (argtype argflag argname)
                     (when (flag-set-p argflag flag-output)
                       (unless (eq (ctype-type argtype) 'C-PTR)
                         (error (TEXT "~S: :OUT argument is not a pointer: ~S")
                                'DEF-CALL-IN argtype))
-                      (format *coutput-stream*
-                              "  ~Aconvert_to_foreign(~A,~A,~A,&~A);~%"
+                      (format *coutput-stream* "  ~A~A;~%"
                               (if (eql outargcount 0) ""
                                 (format nil "if (mv_count >= ~D) "
                                         (+ outargcount 1)))
-                              (object-to-c-value
-                                (pass-object (svref argtype 1)))
-                              (if (eql outargcount 0)
-                                "value1"
-                                (format nil "mv_space[~D]" outargcount))
-                              argname (convert-to-foreign-C argflag))
+                              (convert-to-foreign
+                               (svref argtype 1)
+                               (if (eql outargcount 0) "value1"
+                                   (format nil "mv_space[~D]" outargcount))
+                               argname argflag))
                       (incf outargcount)))
                 argtypes argflags argnames))
         (format *coutput-stream* "  end_callback();~%")
@@ -1215,22 +1263,39 @@
 
 ;; ===========================================================================
 
+(defun form-1+ (form name)
+  "if form is a number, return a number, otherwise return `(1+ ,form)"
+  (typecase form
+    (number (1+ form))
+    (symbol `(1+ ,form))
+    (cons (case (first form)
+            (1+ `(+ 2 ,@(rest form)))
+            (+ (if (numberp (second form))
+                   `(+ ,(1+ (second form)) ,@(cddr form))
+                   `(+ 1 ,@(rest form))))
+            (t `(1+ ,form))))
+    (t (error "~S(~S): invalid value ~S" 'def-c-enum name form))))
+
 (defmacro def-c-enum (&whole whole-form name &rest items)
   (setq name (check-symbol name (first whole-form)))
-  (let ((forms '()) (ht (make-hash-table :key-type 'fixnum :value-type 'symbol))
-        (next-value 0) (this-val 0))
+  (let ((forms '()) (next-value 0) (this-val 0)
+        (ht (make-hash-table :test 'equal :key-type 'fixnum
+                             :value-type 'symbol)))
     (dolist (item items)
       (when (consp item)
         (when (rest item)
-          (setq next-value (second item)
-                this-val (second item)))
+          (let ((value (second item)))
+            (when (constantp value)
+              (setq value (eval value)))
+            (setq next-value value
+                  this-val value)))
         (setq item (first item)))
       (push `(DEFCONSTANT ,item ,next-value) forms)
       (when (gethash this-val ht)
         (warn (TEXT "~S (~S): value ~S will be assigned to both ~S and ~S")
               'def-c-enum name this-val (gethash this-val ht) item))
       (setf (gethash this-val ht) item)
-      (setq next-value `(1+ ,item) this-val (1+ this-val)))
+      (setq next-value `(1+ ,item) this-val (form-1+ this-val name)))
     `(PROGN ,@(nreverse forms) (setf (get ',name 'def-c-enum) ,ht)
             (def-c-type ,name int))))
 
